#!/usr/bin/env ruby

require 'tempfile'
require 'ostruct'
require 'optparse'

def parse(argv)
  options           = OpenStruct.new
  options.processes = []
  options.outfile   = STDOUT

  OptionParser.accept(File, /^(\S+.*)$/ ) do |file|
    if File.file?(file)
      File.open(File.expand_path(file),"w")
    else
      File.open(File.expand_path(file),"w")
    end
  end

  op = OptionParser.new { |opts|
    opts.banner = "Usage: #{$0} [options]"
    opts.separator ""
    opts.separator "Specific options:"
    opts.on("-p", "--process PROCESS", String, "Process to capture" ) { |process|
      begin
        options.processes.push( `#{process} 2>/dev/null` )
      rescue Exception => e
        puts "Can't error processing #{process}: #{e}"
      end
    }

    opts.on("-o" , "--outfile OUTFILE" , File, "Output file" ) do |file|
      options.outfile = file
    end

  }
  begin
    op.parse!(argv)
    raise OptionParser::ParseError.new("--process not specified")  if options.processes.empty?
  rescue OptionParser::ParseError => e
    STDERR.puts e.message, "\n", op
    exit(-1)
  end
  options
end

opts = parse(ARGV)

tmp = Tempfile.new("mytar")
tmp.write( opts.processes[0] )
tmp.close


for i in opts.processes[1..opts.processes.length-1]
  # puts "Found a process"
  tmp2 = Tempfile.new("mytar2")
  tmp2.write( i )
  tmp2.close
  `tar -Af #{tmp.path} #{tmp2.path}`
  tmp2.unlink
end

#puts "outfile: #{opts.outfile}"
opts.outfile.write( `cat #{tmp.path}` )

# system "cp #{tmp.path} #{opts.outfile}"
