#!/bin/bash


#sudo insmod  comedi/drivers/accesisa.ko 

export COMEDI_DEV_ROOT=$(pwd)
export COMEDI_GIT=comedi
export COMEDI_LIB=${COMEDI_DEV_ROOT}/comedilib
export COMEDI_DEMO=${COMEDI_LIB}/demo/


export PATH=$PATH:${COMEDI_DEMO}:${COMEDI_LIB}/comedi_board_info:${COMEDI_LIB}/comedi_config:

COMEDI_DRIVERS=${COMEDI_DEV_ROOT}/${COMEDI_GIT}/comedi/drivers/
BUILT_COMEDI=$(pwd)/.built_comedi
BUILT_COMEDI_LIB=$(pwd)/.built_comedilib
RECONFIGURED_COMEDI=${COMEDI_DEV_ROOT}/.reconfigured_comedi

INSTALL_ROOT=${HOME}/Tools/
COMEDILIB_INSTALL_DIRECTORY=${INSTALL_ROOT}/Comedilib/comedilib_git
export LD_LIBRARY_PATH=$LD_LIBRARY_PATH:${COMEDI_DEV_ROOT}/${COMEDI_LIB}/lib/.libs



function load_comedi_test() { 
    sudo insmod ${COMEDI_DEV_ROOT}/${COMEDI_GIT}/comedi/comedi.ko comedi_num_legacy_minors=8        
    sudo insmod ${COMEDI_DEV_ROOT}/${COMEDI_GIT}/comedi/drivers/comedi_fc.ko 
    sudo insmod ${COMEDI_DEV_ROOT}/${COMEDI_GIT}/comedi/drivers/comedi_test.ko 
    sudo /usr/sbin/comedi_config -v /dev/comedi0_subd1 comedi_test --read-buffer 100 --write-buffer 100 20000,20000
    sudo chmod a+rwx /dev/comedi0
}

function unload_comedi_test() { 
    sudo rmmod  comedi_test comedi_fc comedi 
}

function test_command {
    "$@"
    local status=$?
    if [ $status -ne 0 ]; then
        echo "error with $1" >&2
    fi
    return $status
}

function package_it {
    branch=$1
    download_comedi
    download_comedilib
    ./bin/package_this ${branch}
}


#=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
# Builds latest git comedi_lib unless ${BUILT_COMEDI_LIB} exists
#=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
function build_comedilib() {
    cd ${COMEDI_DEV_ROOT}
    if [ ! -f "${BUILT_COMEDI_LIB}" ] ; then

        download_comedilib

        if [ "$?" != "0" ] ; then
            # return $?
            echo -ne "Do you wish to rebuild comedilib [Y/n] "
            read -n 1 resp            
            case $resp in 
                "y"|"Y"|"")
                    echo  -ne "Rebuilding comedlib\n"
                    ;;
                "n"|"N")
                    echo  -ne "\nNot rebuilding comedilib\n"
                    touch ${BUILT_COMEDI_LIB}
                    return 1
                    ;;
            esac
        fi
        
        rebuild_comedilib

        if [ "$?" == "0" ] ; then
            cd ${COMEDI_DEV_ROOT}
            touch ${BUILT_COMEDI_LIB}
        fi
    else 
        echo "${COMEDI_LIB} has already been built...skipping"
    fi
    cd ${COMEDI_DEV_ROOT}
}

function install_comedi_lib() {
    build_comedilib
    cd ${COMEDI_LIB}
    make install 
    cd -
}

function download_comedi {
    if [ -d ${COMEDI_GIT} ] ; then
        echo -ne "Directory ${COMEDI_GIT} exists, do you want to delete it [N/y] "
        read -n 1 resp
        case $resp in 
            "y"|"Y")
                echo  -ne "\nDeleting comedilib\n"
                rm -rf ${COMEDI_GIT}
                ;;
            "n"|"N"|"")
                echo  -ne "\nNot deleting comedi..."
                return 1
                ;;
        esac
    fi
    if [ ! -z "${USE_FTP_COMEDI}" ] ; then
        wget -O comedigit.tar.gz "http://comedi.org/git?p=comedi/comedi.git;a=snapshot;h=HEAD;sf=tgz"
        tar -xvf comedigit.tar.gz
        if [ -d ${COMEDI_GIT} ] ; then
            rm -rf ${COMEDI_GIT}
        fi
        mv comedi-HEAD-*  ${COMEDI_GIT}
    else 
        test_command git clone git://comedi.org/git/comedi/comedi.git

        if [ ! -z "${USE_COMEDI_BRANCH}" ] ; then
            test_command git checkout -b br_${USE_COMEDI_BRANCH} ${USE_COMEDI_BRANCH}
        fi

    fi

}

function download_comedilib() {
    if [ -d ${COMEDI_LIB} ] ; then
        echo -ne "Directory ${COMEDI_LIB} exists, do you want to delete it [N/y] "
        read -n 1 resp
        case $resp in 
            "y"|"Y")
                echo  -ne "\nDeleting comedilib\n"
                rm -rf ${COMEDI_LIB}
                ;;
            "n"|"N"|"")
                echo  -ne "\nNot deleting comedilib..."
                return 1
                ;;
        esac
    fi

    if [ ! -z "${USE_FTP_COMEDILIB}" ] ; then
        wget -O comedilib.tar.gz "http://comedi.org/git?p=comedi/comedilib.git;a=snapshot;h=HEAD;sf=tgz"
        tar -xvf comedilib.tar.gz
        if [ -d "${COMEDI_LIB}" ] ; then
            rm -rf ${COMEDI_LIB}
        fi
        mv comedilib-HEAD* ${COMEDI_LIB}

    else
        git clone git://comedi.org/git/comedi/comedilib.git comedilib

        if [ ! -z "${USE_COMEDILIB_BRANCH}" ] ; then
            test_command git checkout -b br_${USE_COMEDILIB_BRANCH} ${USE_COMEDILIB_BRANCH}
        fi

    fi
}


#=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
# Builds latest git comedi_lib unless ${BUILT_COMEDI} exists
#=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
function build_comedi() {
    arg=$1
    cd ${COMEDI_DEV_ROOT}
    if [[ ! -f "${BUILT_COMEDI}" || "${arg}" == "-f" ]] ; then

        download_comedi

        if [ "$?" != "0" ] ; then
            # return $?
            echo -ne "Do you wish to rebuild comedi [Y/n] "
            read -n 1 resp            
            case $resp in 
                "y"|"Y"|"")
                    echo  -ne "Rebuilding comedi\n"
                    ;;
                "n"|"N")
                    echo  -ne "\nNot rebuilding comedi\n"
                    touch ${BUILT_COMEDI}
                    return 1
                    ;;
            esac
        fi
        
        rebuild_comedi

        if [ "$?" == "0" ] ; then
            cd ${COMEDI_DEV_ROOT}
            touch ${BUILT_COMEDI}
        fi
    else 
        echo "${COMEDI_LIB} has already been built...skipping"
    fi
    cd ${COMEDI_DEV_ROOT}

}

function setup_acces_kbuild() {
    if [[ -h "${COMEDI_DEV_ROOT}/${COMEDI_GIT}/comedi/drivers/Kbuild" && 
          $(readlink "${COMEDI_DEV_ROOT}/${COMEDI_GIT}/comedi/drivers/Kbuild") == "${COMEDI_DEV_ROOT}/drivers/Kbuild"
                ]] ; then
        echo "Kbuild setup...skipping"
    else 
        echo "Using custom ACCES Kbuild"
        mv -f "${COMEDI_DEV_ROOT}/${COMEDI_GIT}/comedi/drivers/Kbuild" "${COMEDI_DEV_ROOT}/${COMEDI_GIT}/comedi/drivers/Kbuild.bak"
        ln -s "${COMEDI_DEV_ROOT}/drivers/Kbuild" "${COMEDI_DEV_ROOT}/${COMEDI_GIT}/comedi/drivers/Kbuild"
    fi
}

function setup_acces_cfiles() {
    cd "${COMEDI_DEV_ROOT}/${COMEDI_GIT}/comedi/drivers"
    mv -f comedi_test.c comedi_test.c.bak
    ln -f -s ${COMEDI_DEV_ROOT}/drivers/*.{c,h} .
    cd -
}

function reconfigure_comedi() {
    if [ -f "${RECONFIGURED_COMEDI}" ] ; then
        echo "Already reconfigured comedi...skipping"
    else
        echo "Reconfigure comedi"
        cd ${COMEDI_DEV_ROOT}/${COMEDI_GIT}
        configure
        cd -
        touch ${RECONFIGURED_COMEDI}
    fi
}

#=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
# Load the temp accesisa
#=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
function build_acces() { 
    dir=$(pwd)
    build_comedi
    build_comedilib
    setup_acces_kbuild
    setup_acces_cfiles
    reconfigure_comedi

    cd ${COMEDI_DEV_ROOT}/${COMEDI_GIT}/comedi/drivers

    echo "EXTRA CFLAGS=${EXTRA_CFLAGS}!!!\n"
    if [ "${EXTRA_FLAGS}" == "" ] ; then
        ecflags="EXTRA_CFLAGS="${EXTRA_CFLAGS}""
    else 
        ecflags=""
    fi
    
    cmd="make ${verbose} -I${COMEDI_DEV_ROOT}/${COMEDI_GIT}/comedi -C /lib/modules/$(uname -r)/build M=${COMEDI_DEV_ROOT}/${COMEDI_GIT}/comedi CC=\"gcc -I${COMEDI_DEV_ROOT}/${COMEDI_GIT}/include -I${COMEDI_DEV_ROOT}/${COMEDI_GIT}/include -I${COMEDI_DEV_ROOT}/${COMEDI_GIT}/inc-wrap \" $ecflags modules"
    echo "Running '${cmd}'"
    eval $cmd

    cd ${dir}
}

function rebuild_comedilib {
    cd ${COMEDI_LIB}
    sh autogen.sh
    ./configure  --prefix=${COMEDILIB_INSTALL_DIRECTORY} CFLAGS="-g "
    make 
    cd -
}

function rebuild_comedi {
    cd ${COMEDI_GIT}
    sh autogen.sh
    ./configure
    make 
    cd -
}


function rebuild_acces() { 
    cd ${COMEDI_DEV_ROOT}/${COMEDI_GIT}/comedi/drivers
    rm acces*.o 
    rm acces*.ko
    rm acces*.mod.*
    cd -
    build_acces
}


function refresh_build() {
    cd ${COMEDI_DEV_ROOT}/${COMEDI_GIT}
    echo "Reconfiguring"
    configure > log.$(date '+%Y%m%d%H%M%S' )
    cd -
}

function clean_acces() {
    cd ${COMEDI_DRIVERS} 
    rm -f acces*.mod.*
    rm -f acces*.ko
    rm -f acces*.o
    cd -
    rm -f ${BUILT_COMEDI}
    rm -f ${BUILT_COMEDI_LIB}
    rm -f ${RECONFIGURED_COMEDI}
    rm *.tar.gz
}


function load_custom_module() { 
    module=$1
    sudo insmod ${COMEDI_DEV_ROOT}/${COMEDI_GIT}/comedi/comedi.ko comedi_num_legacy_minors=8        
    sudo insmod ${COMEDI_DEV_ROOT}/${COMEDI_GIT}/comedi/drivers/${module}.ko 
    sudo /usr/sbin/comedi_config -v /dev/comedi0 accesio_idio16 0x300,0,0
    sudo chmod a+rwx /dev/comedi0
}

function mload() {
    module=$1
    name=$2
    sudo insmod ${COMEDI_DEV_ROOT}/${COMEDI_GIT}/comedi/comedi.ko
    sudo insmod ${COMEDI_DEV_ROOT}/${COMEDI_GIT}/comedi/drivers/${module}.ko 
    sudo /usr/sbin/comedi_config -v /dev/comedi0_subd1 --read-buffer 1000
    grep -P "$module\s+$name" /proc/comedi > /dev/null
    if [ "$?" == "0" ] ; then
        export LAST_APCI_MODULE=${module}
        export LAST_MODULE_NAME=${name}
    else
        echo "An error occurred...."
        sudo rmmod ${module} comedi 
    fi
}

function munload() {
    module=$1
    if [ ! -z $module ] ; then 
        sudo rmmod $module comedi
    elif [ "$LAST_APCI_MODULE" != "" ] ; then
        sudo rmmod $LAST_APCI_MODULE comedi
        if [ "$?" == "0" ] ; then
            unset LAST_APCI_MODULE
            unset LAST_MODULE_NAME
        fi
    else

        sudo rmmod comedi_fc comedi
        if [ "$?" == "0" ] ; then
            unset LAST_APCI_MODULE
        fi

    fi
}

function load_acces() { 
    sudo insmod ${COMEDI_DEV_ROOT}/${COMEDI_GIT}/comedi/comedi.ko comedi_num_legacy_minors=8        
    sudo insmod ${COMEDI_DEV_ROOT}/${COMEDI_GIT}/comedi/drivers/accesisa.ko 
    sudo /usr/sbin/comedi_config -v /dev/comedi0 accesio_idio16 0x300,0,0
    sudo chmod a+rwx /dev/comedi0
}

function unload_acces() { 
    sudo rmmod accesisa comedi
}

function try_build() { 
    make -I/home/jdamon/Projects/OnGoing/ACCES/first_comedi_driver/${COMEDI_GIT}/comedi -C /lib/modules/3.5.0-52-generic/build M=/home/jdamon/Projects/OnGoing/ACCES/first_comedi_driver/${COMEDI_GIT}/comedi CC="gcc -I/home/jdamon/Projects/OnGoing/ACCES/first_comedi_driver/${COMEDI_GIT}/ -I/home/jdamon/Projects/OnGoing/ACCES/first_comedi_driver/${COMEDI_GIT}/include  " modules
}

function retry_acces() { 
    unload_acces
    try_build
    load_acces
}

function demo_it() { 
    cd comedilib/demo
    cmd -v -p -n 2 -N 40
    cd -
}
