/**
 * Example for testing the PC104idio
 *
 */

#include <stdio.h>
#include <comedilib.h>
#include <fcntl.h>
#include <unistd.h>
#include <sys/time.h>
#include <errno.h>
#include <stdlib.h>
#include <string.h>
#include "gtest/gtest.h"
#include "tap.h"

#include <sys/io.h>


extern comedi_t *device;

struct parsed_options
{
    char *filename;
    double value;
    int subdevice;
    int subdevice2;
    int channel;
    int aref;
    int range;
    int verbose;
    int n_chan;
    int n_scan;
    double freq;
    int dosomething;
};



/* #define BUFSZ 10000 */
/* char buf[BUFSZ]; */

#define N_CHANS 256
static unsigned int chanlist[N_CHANS];
static comedi_range * range_info[N_CHANS];
static lsampl_t maxdata[N_CHANS];


int prepare_cmd_lib(comedi_t *dev, int subdevice, int n_scan, int n_chan, unsigned period_nanosec, comedi_cmd *cmd);

void do_cmd(comedi_t *dev,comedi_cmd *cmd);

void print_datum(lsampl_t raw, int channel_index);

char *cmdtest_messages[]={
    (char *)"success",
    (char *)"invalid source",
    (char *)"source conflict",
    (char *)"invalid argument",
    (char *)"argument conflict",
    (char *)"invalid chanlist",
};
struct parsed_options options;
int subdev_flags;
comedi_t *dev;
comedi_cmd c,*cmd;
lsampl_t raw;
#define BUFSZ 1024
sampl_t buf[BUFSZ];

TEST(ComediBasic,AbleToDoSomething) {

}

#define COMEDI_DATA_READ(...) ret = comedi_data_read( __VA_ARGS__ );\
    ASSERT_TRUE(ret);
#define COMEDI_DIO_BITFIELD(...)  ret  = comedi_dio_bitfield( __VA_ARGS__);\
    ASSERT_TRUE( ret >= 0 );

void toggle_interrupt()
{
    outb(0x1, 0x302 );          /* temp disable */
    outb(0x1, 0x301);           /* clear intterrupt */
    inb(0x302);
    inl(0x302 );
    usleep(100);
}

void start_waiting_for_interrupt()
{
    outb(0x1, 0x302 );          /* temp disable */
    outb(0x1, 0x301);           /* clear intterrupt */
    inb(0x302);
}

void stop_waiting_for_interrupt()
{
  outb(0x1, 0x302 );
}

TEST(ComediBasic,TestPoll ) {
    lsampl_t data;
    int ret;
    int subdev = 0;        
    int chan = 0;          
    int range = 0;         
    int aref = AREF_GROUND;
    fd_set rdset;
    struct timeval timeout;
    char buf[1024];
    dev = comedi_open(options.filename);
    ASSERT_TRUE( dev );
    /* ret = comedi_poll(dev,0); */
    FD_ZERO(&rdset);
    FD_SET(comedi_fileno(dev),&rdset);
    timeout.tv_sec = 0;
    timeout.tv_usec = 50000;
    /* ret = select(comedi_fileno(dev)+1,&rdset,NULL,NULL,&timeout); */
    /* ret=read(comedi_fileno(dev),buf,sizeof(buf)); */
    ret = comedi_data_read( dev, 0, 0, 0, 0, (lsampl_t*)buf );

    EXPECT_GE(ret,0);
}


TEST(ComediBasic,TestIrq ) {
    lsampl_t data;
    int ret;
    int subdev = 0;        
    int chan = 0;          
    int range = 0;         
    int aref = AREF_GROUND;

    dev = comedi_open(options.filename);
    ASSERT_TRUE( dev );
    subdev_flags = comedi_get_subdevice_flags(dev, options.subdevice);
    start_waiting_for_interrupt();
    for( int i = 0 ; i < 500 ; i ++ ) {
      comedi_dio_bitfield2( dev, options.subdevice, 0xffff, &data, 0 );
      toggle_interrupt();
      usleep(100);
    }
    comedi_close( dev );

    printf("%d\n", inb(0x305 ));

}


/**
 * This test proves that we can read from the card.
 * Now, we want a way to verify that we can actually write to the card too !
 */
TEST(ComediBasic,Test_read_auto_incrementing_digital ) {
    lsampl_t data;
    int ret;
    int subdev = 0;        
    int chan = 0;          
    int range = 0;         
    int aref = AREF_GROUND;

    dev = comedi_open(options.filename);
    ASSERT_TRUE( dev );
    subdev_flags = comedi_get_subdevice_flags(dev, options.subdevice);
    /* ret = comedi_dio_config( dev, options.subdevice, options.channel, COMEDI_INPUT ); */
    /* only necessary if the device is d/io */
    /* ASSERT_TRUE( ret >= 0 ); */

    /* COMEDI_DIO_BITFIELD(dev,options.subdevice, 0, &data); */
    inb(0x301);
    outb(0xfe,0x300);
    comedi_dio_bitfield2( dev, options.subdevice, 0xffff, &data, 0 );
    EXPECT_EQ( data, 0x00 );
    COMEDI_DIO_BITFIELD(dev,options.subdevice, 0, &data);
    EXPECT_EQ( data, 0x01 );
    COMEDI_DIO_BITFIELD(dev,options.subdevice, 0, &data);
    EXPECT_EQ( data, 0x02 );
    comedi_close( dev );
}

#define COMEDI_DIO_BITFIELD2(...)   ret  = comedi_dio_bitfield2( __VA_ARGS__); \
    ASSERT_TRUE( ret >= 0 );


TEST(ComediBasic,Test_writing_digital ) {
    lsampl_t data;
    int ret;
    int subdev = 0;        
    int chan = 0;          
    int range = 0;         
    int aref = AREF_GROUND;

    dev = comedi_open(options.filename);
    ASSERT_TRUE( dev );
    subdev_flags = comedi_get_subdevice_flags(dev, options.subdevice);
    ret = comedi_dio_config( dev, options.subdevice, options.channel, COMEDI_OUTPUT );
    ASSERT_TRUE( ret >= 0 );
    data = 0x0;
    COMEDI_DIO_BITFIELD2(dev,options.subdevice, 0, &data , 0);
    /* EXPECT_EQ( data, 0x00 ); */
    data = 0x1;
    COMEDI_DIO_BITFIELD2(dev,options.subdevice, 0, &data , 0);
    /* EXPECT_EQ( data, 0x01 ); */
    data = 0x23;
    COMEDI_DIO_BITFIELD2(dev,options.subdevice, 0, &data , 0);
    /* EXPECT_EQ( data, 0x02 ); */
    comedi_close( dev );
}


TEST(ComediBasic,TestDigital) {
    int stype;
    dev = comedi_open(options.filename);
    ASSERT_TRUE( dev );
    stype = comedi_get_subdevice_type(dev, options.subdevice);
    EXPECT_TRUE( stype >= 0 );
    /* printf("value returned was %d\n", stype ); */
    stype = comedi_get_subdevice_type(dev, options.subdevice2);
    EXPECT_TRUE( stype >= 0 );
    /* printf("value returned was %d\n", stype ); */

}


TEST(ComediBasic,TestDigital_again) {
    /* init_parsed_options(&options); */
    /* parse_options(&options, argc, argv); */
    int stype;
}

                                          
int main(int argc, char *argv[])
{
	int ret;
	int total=0;
	int i;
        cmd =&c;
        /* init_parsed_options( &options ); */

        if ( iopl(3) < 0 ) {
          fprintf(stderr,"No Permission\n");
          exit(1);
        }
        outl(0,0x302 );
	memset(&options, 0, sizeof(options));
	/* The following variables used in this demo
	 * can be modified by command line
	 * options.  When modifying this demo, you may want to
	 * change them here. */
	options.filename = (char *)"/dev/comedi0";
	options.subdevice = 0;
	options.subdevice2 = 1;
	options.channel = 1;
	options.range = 0;
	options.aref = AREF_GROUND;
	options.n_chan = 4;
	options.n_scan = 10000;
	options.freq = 5000.0;
        /* parse_options( &options, argc, argv ); */
	/* open the device */
	/* dev = comedi_open(options.filename); */
        ::testing::GTEST_FLAG(print_time) = false;

        testing::InitGoogleTest(&argc, argv);
        testing::TestEventListeners & listeners = testing::UnitTest::GetInstance()->listeners();
#ifdef GTEST_TAP_PRINT_TO_STDOUT
        delete listeners.Release(listeners.default_result_printer());
#endif
        
        listeners.Append( new tap::TapListener() );
        return RUN_ALL_TESTS();  



	return 0;
}

/*
 * This prepares a command in a pretty generic way.  We ask the
 * library to create a stock command that supports periodic
 * sampling of data, then modify the parts we want. */
int prepare_cmd_lib(comedi_t *dev, int subdevice, int n_scan, int n_chan, unsigned scan_period_nanosec, comedi_cmd *cmd)
{
	int ret;

	memset(cmd,0,sizeof(*cmd));

	/* This comedilib function will get us a generic timed
	 * command for a particular board.  If it returns -1,
	 * that's bad. */
	ret = comedi_get_cmd_generic_timed(dev, subdevice, cmd, n_chan, scan_period_nanosec);
	if (ret<0){
		printf("comedi_get_cmd_generic_timed failed\n");
		return ret;
	}

	/* Modify parts of the command */
	cmd->chanlist = chanlist;
	cmd->chanlist_len = 1;
	if (cmd->stop_src == TRIG_COUNT) cmd->stop_arg = n_scan;

	return 0;
}

void print_datum(lsampl_t raw, int channel_index) {
	double physical_value;
	physical_value = comedi_to_phys(raw, range_info[channel_index], maxdata[channel_index]);
	printf("%#8.6g ",physical_value);
}
